@@ -41,6 +41,7 @@
 gboolean silent_errors = FALSE;
 
 GDBusProxy *powerd_proxy = NULL;
+GDBusProxy *uscreen_proxy = NULL;
 char test_dbusname[128] = "";
 
 /* For tests */
@@ -53,13 +54,6 @@ struct PublicSysRequest {
     int state;
 };
 
-struct PublicDispRequest {
-    const char *name;
-    const char *owner;
-    enum powerd_display_state state;
-    guint32 flags;
-};
-
 struct SysRequestStats {
     const char *owner;
     const char *name;
@@ -69,19 +63,6 @@ struct SysRequestStats {
     guint64 active_since;
 };
 
-struct DispRequestStats {
-    const char *owner;
-    const char *name;
-    unsigned active_count;
-    guint64 active_time;
-    guint64 max_active_time;
-    guint64 active_since;
-    guint64 disp_on_time;
-    guint64 disp_on_since;
-    guint64 flag_on_time[POWERD_NUM_DISPLAY_FLAGS];
-    guint64 flag_on_since[POWERD_NUM_DISPLAY_FLAGS];
-};
-
 static GMainLoop *main_loop = NULL;
 static powerd_cookie_t main_cookie;
 static gboolean checkForDbusName(const char *dbusname, int count,
@@ -119,6 +100,32 @@ on_powerd_signal (GDBusProxy *proxy,
     }
 }
 
+gboolean
+requestProximityBlanking(gboolean enable)
+{
+    GVariant *ret = NULL;
+    GError *error = NULL;
+    const char *method = enable ?
+        "enableProximityHandling":
+        "disableProximityHandling";
+
+    ret = g_dbus_proxy_call_sync(powerd_proxy,
+            method,
+            g_variant_new("(s)", "powerd-cli"),
+            G_DBUS_CALL_FLAGS_NONE,
+            -1,
+            NULL,
+            &error);
+    if (ret == NULL) {
+        cli_warn("%s failed: %s", method, error->message);
+        g_error_free(error);
+        return FALSE;
+    }
+
+    g_variant_unref(ret);
+    return TRUE;
+}
+
 gboolean
 requestSysState(const char *name, int state,
         powerd_cookie_t *cookie)
@@ -160,22 +167,69 @@ requestSysState(const char *name, int state,
 }
 
 static gboolean
-requestDisplayState(struct PublicDispRequest pdr, const char *name,
-        powerd_cookie_t *cookie)
+requestWakeup(const char *name,
+              uint64_t wakeup_time,
+              powerd_cookie_t setme_cookie)
+{
+    GVariant *ret;
+    GError *error;
+    gboolean success;
+
+    g_return_val_if_fail (setme_cookie != NULL, FALSE);
+
+    error = NULL;
+    ret = g_dbus_proxy_call_sync(powerd_proxy,
+                                 "requestWakeup",
+                                 g_variant_new("(st)", name, wakeup_time),
+                                 G_DBUS_CALL_FLAGS_NONE,
+                                 -1, /* use default timeout */
+                                 NULL, /* cancellable */
+                                 &error);
+
+    if (ret == NULL) {
+        cli_warn("%s failed: %s", G_STRFUNC, error->message);
+        success = FALSE;
+    }
+    else if (!g_variant_is_of_type(ret, G_VARIANT_TYPE("(s)"))) {
+        char * s = g_variant_print(ret, TRUE);
+        cli_warn("Unexpected response: %s", s);
+        g_free(s);
+        success = FALSE;
+    }
+    else {
+        const char* s = NULL;
+        g_variant_get(ret, "(&s)", &s);
+        if (strlen(s) != sizeof(powerd_cookie_t) - 1) {
+            cli_warn("Returned cookie has incorrect size");
+            success = FALSE;
+        }
+        else {
+            memcpy(setme_cookie, s, sizeof(powerd_cookie_t));
+            cli_debug("Got cookie: %s", setme_cookie);
+            success = TRUE;
+        }
+    }
+
+    g_clear_error(&error);
+    g_clear_pointer(&ret, g_variant_unref);
+    return success;
+}
+
+static gboolean
+requestDisplayState(int *cookie)
 {
     GVariant *ret = NULL;
     GError *error = NULL;
-    const char *cookie_ptr;
     gboolean success = TRUE;
 
     if (cookie == NULL) {
-        cli_warn("NULL cookie passed to %s", __func__);
+        cli_warn("NULL cookie store passed to %s", __func__);
         return FALSE;
     }
 
-    ret = g_dbus_proxy_call_sync(powerd_proxy,
-            "requestDisplayState",
-            g_variant_new("(siu)", name, pdr.state, pdr.flags),
+    ret = g_dbus_proxy_call_sync(uscreen_proxy,
+            "keepDisplayOn",
+            NULL,
             G_DBUS_CALL_FLAGS_NONE,
             -1,
             NULL,
@@ -186,44 +240,8 @@ requestDisplayState(struct PublicDispRequest pdr, const char *name,
         return FALSE;
     }
 
-    g_variant_get(ret, "(&s)", &cookie_ptr);
-    if (strlen(cookie_ptr) != sizeof(powerd_cookie_t) - 1) {
-        cli_warn("Returned cookie has incorrect size");
-        success = FALSE;
-    } else {
-        strncpy(*cookie, cookie_ptr, sizeof(powerd_cookie_t));
-        cli_debug("Got cookie: %s", *cookie);
-    }
-
-    g_variant_unref(ret);
-    return success;
-}
-
-static gboolean
-updateDisplayState(struct PublicDispRequest pdr,
-        powerd_cookie_t cookie)
-{
-    GVariant *ret = NULL;
-    GError *error = NULL;
-    gboolean success = TRUE;
-
-    if (cookie == NULL) {
-        cli_warn("NULL cookie passed to %s", __func__);
-        return FALSE;
-    }
-
-    ret = g_dbus_proxy_call_sync(powerd_proxy,
-            "updateDisplayState",
-            g_variant_new("(siu)", cookie, pdr.state, pdr.flags),
-            G_DBUS_CALL_FLAGS_NONE,
-            -1,
-            NULL,
-            &error);
-    if (!ret) {
-        cli_warn("updateDisplayState failed: %s", error->message);
-        g_error_free(error);
-        return FALSE;
-    }
+    g_variant_get(ret, "(i)", cookie);
+    cli_debug("Got cookie: %d", *cookie);
 
     g_variant_unref(ret);
     return success;
@@ -280,58 +298,6 @@ printSysRequests(GArray *requests)
     }
 }
 
-static GArray*
-listDisplayRequests()
-{
-    GVariant *ret, *item = NULL;
-    GVariantIter *iter = NULL;
-    GArray *retarray = g_array_new(FALSE, FALSE, sizeof(struct PublicDispRequest));
-    GError *error = NULL;
-    struct PublicDispRequest pdr;
-
-    ret = g_dbus_proxy_call_sync(powerd_proxy,
-            "listDisplayRequests",
-            NULL,
-            G_DBUS_CALL_FLAGS_NONE,
-            -1,
-            NULL,
-            &error);
-    if (ret == NULL) {
-        cli_warn("listDisplayRequests failed: %s", error->message);
-        g_error_free(error);
-    }
-    else {
-        g_variant_get(ret, "(a(ssiu))", &iter);
-        while ((item = g_variant_iter_next_value (iter))) {
-            g_variant_get_child(item, 0, "s", &pdr.name);
-            g_variant_get_child(item, 1, "s", &pdr.owner);
-            g_variant_get_child(item, 2, "i", &pdr.state);
-            g_variant_get_child(item, 3, "u", &pdr.flags);
-            g_array_append_val(retarray, pdr);
-            g_variant_unref(item);
-        }
-        g_variant_unref(ret);
-    }
-    return retarray;
-}
-
-static void
-printDisplayRequests(GArray *requests)
-{
-    int i;
-    struct PublicDispRequest *pdr;
-    printf("Display State Requests:\n");
-    if (requests->len == 0) {
-        printf("  None\n");
-    } else {
-        for (i = 0; i < requests->len; i++) {
-            pdr = &g_array_index(requests, struct PublicDispRequest, i);
-            printf("  Name: %s, Owner: %s, State: %d, Flags: %#08x\n",
-                   pdr->name, pdr->owner, pdr->state, pdr->flags);
-        }
-    }
-}
-
 static GArray *
 getSysRequestStats(void)
 {
@@ -395,106 +361,33 @@ printSysRequestStats(GArray *stats)
     }
 }
 
-static GArray *
-getDispRequestStats(void)
+static gboolean
+clearWakeup(const char* cookie_str)
 {
-    GVariant *ret, *item;
-    GVariantIter *iter;
-    GArray *retarray;
+    GVariant *ret;
     GError *error;
-
-    retarray =  g_array_new(FALSE, FALSE, sizeof(struct DispRequestStats));
+    gboolean success;
 
     error = NULL;
     ret = g_dbus_proxy_call_sync(powerd_proxy,
-            "getDispRequestStats",
-            NULL,
-            G_DBUS_CALL_FLAGS_NONE,
-            -1,
-            NULL,
-            &error);
+                                 "clearWakeup",
+                                 g_variant_new("(s)", cookie_str),
+                                 G_DBUS_CALL_FLAGS_NONE,
+                                 -1, /* default timeout */
+                                 NULL, /* cancellable */
+                                 &error);
+
     if (ret == NULL) {
-        cli_warn("getDispRequestStats failed: %s", error->message);
-        g_error_free(error);
+        cli_warn("%s failed: %s", G_STRFUNC, error->message);
+        success = FALSE;
     } else {
-        g_variant_get(ret, "(a(ssutttttatat))", &iter);
-        while ((item = g_variant_iter_next_value (iter))) {
-            struct DispRequestStats stats;
-            GVariantIter *array_iter;
-            guint64 val;
-            int i;
-
-            g_variant_get_child(item, 0, "s", &stats.owner);
-            g_variant_get_child(item, 1, "s", &stats.name);
-            g_variant_get_child(item, 2, "u", &stats.active_count);
-            g_variant_get_child(item, 3, "t", &stats.active_time);
-            g_variant_get_child(item, 4, "t", &stats.max_active_time);
-            g_variant_get_child(item, 5, "t", &stats.active_since);
-            g_variant_get_child(item, 6, "t", &stats.disp_on_time);
-            g_variant_get_child(item, 7, "t", &stats.disp_on_since);
-
-            g_variant_get_child(item, 8, "at", &array_iter);
-            i = 0;
-            while (g_variant_iter_loop(array_iter, "t", &val)) {
-                if (i >= POWERD_NUM_DISPLAY_FLAGS)
-                    break;
-                stats.flag_on_time[i++] = val;
-            }
-            g_variant_iter_free(array_iter);
-
-            g_variant_get_child(item, 9, "at", &array_iter);
-            i = 0;
-            while (g_variant_iter_loop(array_iter, "t", &val)) {
-                if (i >= POWERD_NUM_DISPLAY_FLAGS)
-                    break;
-                stats.flag_on_since[i++] = val;
-            }
-            g_variant_iter_free(array_iter);
-
-            g_array_append_val(retarray, stats);
-            g_variant_unref(item);
-        }
-        g_variant_unref(ret);
+        cli_debug("%s success", G_STRFUNC);
+        success = TRUE;
     }
-    return retarray;
-}
 
-static void
-printDispRequestStats(GArray *stats)
-{
-    int i, j;
-    struct DispRequestStats *stat;
-    printf("Display Request Statistics:\n");
-    if (stats->len == 0) {
-        printf("  None\n");
-    } else {
-        printf("  %-16.16s %-20.20s %-8.8s %-16.16s %-16.16s %-16.16s %-16.16s %-16.16s",
-               "", "", "Active", "", "Max Active", "", "Display On",
-               "Display On");
-        for (i = 0; i < POWERD_NUM_DISPLAY_FLAGS; i++)
-            printf(" Flag %-2d          Flag %-2d         ", i, i);
-        printf("\n  %-16.16s %-20.20s %-8.8s %-16.16s %-16.16s %-16.16s %-16.16s %-16.16s",
-               "Owner", "Name", "Count", "Active Time", "Time", "Active Since",
-               "Time", "Since");
-        for (i = 0; i < POWERD_NUM_DISPLAY_FLAGS; i++)
-            printf(" On Time          On Since        ");
-        printf("\n");
-        for (i = 0; i < stats->len; i++) {
-            stat = &g_array_index(stats, struct DispRequestStats, i);
-            printf("  %-16.16s %-20.20s %-8u %-16.6f %-16.6f %-16.6f %-16.6f %-16.6f",
-                   stat->owner, stat->name, stat->active_count,
-                   (double)stat->active_time / 1000000.0f,
-                   (double)stat->max_active_time / 1000000.0f,
-                   (double)stat->active_since / 1000000.0f,
-                   (double)stat->disp_on_time / 1000000.0f,
-                   (double)stat->disp_on_since / 1000000.0f);
-            for (j = 0; j < POWERD_NUM_DISPLAY_FLAGS; j++) {
-                printf(" %-16.6f", (double)stat->flag_on_time[j] / 1000000.0f);
-                printf(" %-16.6f", (double)stat->flag_on_since[j] / 1000000.0f);
-            }
-            printf("\n");
-        }
-    }
+    g_clear_error(&error);
+    g_clear_pointer(&ret, g_variant_unref);
+    return success;
 }
 
 gboolean
@@ -520,20 +413,20 @@ clearSysState(powerd_cookie_t cookie)
 }
 
 static gboolean
-clearDisplayState(powerd_cookie_t cookie)
+clearDisplayState(int cookie)
 {
     GVariant *ret = NULL;
     GError *error = NULL;
 
-    ret = g_dbus_proxy_call_sync(powerd_proxy,
-            "clearDisplayState",
-            g_variant_new("(s)", cookie),
+    ret = g_dbus_proxy_call_sync(uscreen_proxy,
+            "removeDisplayOnRequest",
+            g_variant_new("(i)", cookie),
             G_DBUS_CALL_FLAGS_NONE,
             -1,
             NULL,
             &error);
     if (ret == NULL) {
-        cli_warn("clearDisplayState failed: %s", error->message);
+        cli_warn("removeDisplayOnRequest failed: %s", error->message);
         g_error_free(error);
         return FALSE;
     }
@@ -567,9 +460,6 @@ cb_child_watch(GPid pid,
         requests = listSysRequests();
         do_test(checkForDbusName(test_dbusname, 0, requests, TRUE));
         g_array_free(requests, TRUE);
-        requests = listDisplayRequests();
-        do_test(checkForDbusName(test_dbusname, 0, requests, TRUE));
-        g_array_free(requests, TRUE);
     }
     else {
         printf("Did not find child's dbus name\n");
@@ -712,102 +602,6 @@ runSysTests()
     silence_errors(FALSE);
 }
 
-static void
-runDisplayTests()
-{
-    int i;
-    int state = 0;
-    gboolean bright = FALSE;
-    powerd_cookie_t cookie, cookies[TEST_NUM_DISP_REQUESTS];
-    GArray *requests = NULL;
-    struct PublicDispRequest pdr = {0,};
-
-    silence_errors(TRUE);
-
-    // Hold active state request as long we're running tests
-    requestSysState("main-req", POWERD_SYS_STATE_ACTIVE, &main_cookie);
-
-    for (i = 0; i < TEST_NUM_DISP_REQUESTS; i++) {
-        char name[16];
-        snprintf(name, sizeof(name), "disp-test-%d", i);
-        pdr.state = state++;
-        pdr.flags = 0;
-        if (bright) {
-            pdr.flags |= POWERD_DISPLAY_FLAG_BRIGHT;
-        }
-        do_test(requestDisplayState(pdr, name, &cookies[i]) == TRUE);
-        if (state >= POWERD_NUM_DISPLAY_STATES) {
-            state = 0;
-        }
-        bright = !bright;
-    }
-
-    // Make sure we have at least NUM_REQUESTS
-    requests = listDisplayRequests();
-    do_test(requests->len >= TEST_NUM_DISP_REQUESTS);
-    // We should see our PID NUM_REQUESTS times
-    do_test(checkForDbusName(powerd_cli_bus_name, TEST_NUM_DISP_REQUESTS, requests, FALSE));
-    g_array_free(requests, TRUE);
-
-    for (i = 0; i < TEST_NUM_DISP_REQUESTS; i++)
-        do_test(clearDisplayState(cookies[i]) == TRUE);
-
-    requests = listDisplayRequests();
-    // We aren't holding anymore with this PID
-    do_test(checkForDbusName(powerd_cli_bus_name, 0, requests, FALSE));
-    g_array_free(requests, TRUE);
-
-    // Now try and freeing the same cookies again. They should all be
-    // invalid and thus the request fails.
-    for (i = 0; i < TEST_NUM_DISP_REQUESTS; i++)
-        do_test(clearDisplayState(cookies[i]) == FALSE);
-
-    // Test releasing an invalid cookie format
-    do_test(clearDisplayState("bad cookie") == FALSE);
-
-    //invalid values
-    pdr.state = -1;
-    do_test(requestDisplayState(pdr, "disp-test", &cookie) == FALSE);
-
-    pdr.state = POWERD_NUM_DISPLAY_STATES;
-    do_test(requestDisplayState(pdr, "disp-test", &cookie) == FALSE);
-
-    pdr.state = POWERD_DISPLAY_STATE_ON;
-    pdr.flags = 0xdeadbeef;
-    do_test(requestDisplayState(pdr, "disp-test", &cookie) == FALSE);
-
-    // updateDisplayState tests
-    pdr.state = POWERD_DISPLAY_STATE_ON;
-    pdr.flags = 0;
-    do_test(requestDisplayState(pdr, "disp-test", &cookie) == TRUE);
-
-    // update with same state should be okay
-    do_test(updateDisplayState(pdr, cookie) == TRUE);
-
-    pdr.state = POWERD_DISPLAY_STATE_DONT_CARE;
-    do_test(updateDisplayState(pdr, cookie) == TRUE);
-
-    pdr.flags = POWERD_DISPLAY_FLAG_DISABLE_AUTOBRIGHTNESS;
-    do_test(updateDisplayState(pdr, cookie) == TRUE);
-
-    pdr.state = -1;
-    do_test(updateDisplayState(pdr, cookie) == FALSE);
-
-    do_test(clearDisplayState(cookie) == TRUE);
-
-    // update with same cookie after clear should fail
-    do_test(updateDisplayState(pdr, cookie) == FALSE);
-
-    // update with garbage cookie should faile
-    do_test(updateDisplayState(pdr, "bad cookie") == FALSE);
-
-    //cleanup
-    do_test(clearSysState(main_cookie) == TRUE);
-    do_test(checkForDbusName(powerd_cli_bus_name, 0, requests, TRUE));
-
-    silence_errors(FALSE);
-}
-
 // Check that dbus owner appears count times in a list of sys/disp requests
 static gboolean
 checkForDbusName(const char *dbusname,
@@ -818,7 +612,6 @@ checkForDbusName(const char *dbusname,
     int i;
     int found = 0;
     struct PublicSysRequest *psr;
-    struct PublicDispRequest *pdr;
 
     if (requests == NULL) {
         cli_warn("NULL requests passed to %s", __func__);
@@ -833,12 +626,6 @@ checkForDbusName(const char *dbusname,
                 found++;
             }
         }
-        else {
-            pdr = &g_array_index(requests, struct PublicDispRequest, i);
-            if (!(strcmp(pdr->owner, dbusname))) {
-                found++;
-            }
-        }
     }
 
     if (found != count) {
@@ -855,47 +642,6 @@ sig_handler(int signum)
     g_main_loop_quit(main_loop);
 }
 
-/* Caller ensures that we have at least 4 arguments in argv */
-static gboolean
-buildDisplayRequest(char **argv,
-        int argc,
-        struct PublicDispRequest *pdr)
-{
-    int i;
-
-    if (!strcasecmp(argv[2],"on")) {
-        pdr->state = POWERD_DISPLAY_STATE_ON;
-        cli_debug("Requesting Display On");
-    }
-    else if (!strcasecmp(argv[2],"dc")) {
-        pdr->state = POWERD_DISPLAY_STATE_DONT_CARE;
-        cli_debug("Requesting Display Don't Care");
-    }
-    else {
-        fprintf(stderr,"invalid state %s, must be either on or dc\n",
-            argv[2]);
-        return FALSE;
-    }
-
-    pdr->flags = 0;
-    for (i=3; i<argc; i++) {
-        if (!strcmp(argv[i],"proximity")) {
-            pdr->flags |= POWERD_DISPLAY_FLAG_USE_PROXIMITY;
-            cli_debug("Requesting Proximity Sensor Enabled");
-        }
-        else if (!strcmp(argv[i],"disableab")) {
-            pdr->flags |= POWERD_DISPLAY_FLAG_DISABLE_AUTOBRIGHTNESS;
-            cli_debug("Requesting Proximity Disable AutoBrightness");
-        }
-        else if (!strcasecmp(argv[i],"bright")) {
-            pdr->flags |= POWERD_DISPLAY_FLAG_BRIGHT;
-            cli_debug("Requesting Bright");
-        }
-    }
-
-    return TRUE;
-}
-
 static void
 setUserAutobrightness(gboolean enable)
 {
@@ -916,7 +662,7 @@ setUserAutobrightness(gboolean enable)
 }
 
 static gboolean
-getBrightnessParams(int *min, int *max, int *dflt, gboolean *ab_supported)
+getBrightnessParams(int *dim, int *min, int *max, int *dflt, gboolean *ab_supported)
 {
     GVariant *ret = NULL;
     GError *error = NULL;
@@ -932,7 +678,7 @@ getBrightnessParams(int *min, int *max, int *dflt, gboolean *ab_supported)
         return FALSE;
     }
 
-    g_variant_get(ret, "((iiib))", min, max, dflt, ab_supported);
+    g_variant_get(ret, "((iiiib))", dim, min, max, dflt, ab_supported);
     g_variant_unref(ret);
     return TRUE;
 }
@@ -966,23 +712,20 @@ usage(const char *progname)
             "exit, causing the request to be dropped.\n");
     printf("autobrightness <enable|disable> - enable or disable autobrightness.\n");
     printf("brightness <brightness> - set user screen brightness.\n");
-    printf("brightness-params - get brightness parameters (min, max, etc.)\n");
+    printf("brightness-params - get brightness parameters (dim, min, max, etc.)\n");
     printf("clear-sys <cookie> - clear a System state request given a cookie.\n");
-    printf("clear-disp <cookie> - clear a Display state request given a cookie.\n");
     printf("client-test - test powerd registration / ack API.\n");
-    printf("display <on|dc> [bright] [proximity] [disableab]\n"\
-           "\tMake a display state request with the input parameters.\n"\
-           "\tThe first argument represents the state of the display:\n"\
-           "\tOn (on) or Don't Care (dc),\n"\
-           "\tThe final optional arguments respectively:\n"\
-           "\t * make the screen bright [bright]\n"\
-           "\t * enable the proximity sensor [proximity]\n"\
-           "\t * disable autobrightness [disableab]\n");
+    printf("display <on>\n"\
+           "\tMake the display stay on\n");
+    printf("proximity on|off\n"\
+           "\tEnable disable proximity based screen blanking\n");
     printf("help\t- display this usage information.\n");
     printf("list\t- list outstanding requests.\n");
     printf("listen\t- listen for signals from powerd. This runs a\n"\
            "\t  gmainloop and must be manually killed.\n");
     printf("stats\t- print request statistics.\n");
+    printf("wakeup <n>\t- request a hardware wakeup n seconds from now.\n");
+    printf("clear-wakeup <cookie>\t- clear a hardware wakeup request given a cookie.\n");
     printf("test\t- runs tests.\n");
 }
 
@@ -997,7 +740,6 @@ int main (int argc, char **argv)
     GError *error = NULL;
     uid_t myeuid;
     GArray *requests = NULL;
-    struct PublicDispRequest pdr;
     GDBusConnection *bus = NULL;
     const char *bus_name = NULL;
     powerd_cookie_t cookie;
@@ -1012,12 +754,14 @@ int main (int argc, char **argv)
        (strcmp(argv[1],"stats")) &&
        (strcmp(argv[1],"active")) &&
        (strcmp(argv[1],"active-nc")) &&
+       (strcmp(argv[1],"wakeup")) &&
+       (strcmp(argv[1],"clear-wakeup")) &&
        (strcmp(argv[1],"test")) &&
        (strcmp(argv[1],"dbusnametest")) &&
        (strcmp(argv[1],"listen")) &&
        (strcmp(argv[1],"display")) &&
+       (strcmp(argv[1],"proximity")) &&
        (strcmp(argv[1],"help")) &&
-       (strcmp(argv[1],"clear-disp")) &&
        (strcmp(argv[1],"clear-sys")) &&
        (strcmp(argv[1],"client-test")) &&
        (strcmp(argv[1], "autobrightness")) &&
@@ -1055,22 +799,30 @@ int main (int argc, char **argv)
         return -1;
     }
 
+    uscreen_proxy = g_dbus_proxy_new_for_bus_sync(G_BUS_TYPE_SYSTEM,
+                G_DBUS_PROXY_FLAGS_NONE,
+                NULL,
+                "com.canonical.Unity.Screen",
+                "/com/canonical/Unity/Screen",
+                "com.canonical.Unity.Screen",
+                NULL,
+                &error);
+
+    if (error != NULL) {
+        cli_warn("could not connect to unity screen: %s", error->message);
+        g_error_free(error);
+        return -1;
+    }
+
     if (!strcmp(argv[1],"list")) {
         requests = listSysRequests();
         printSysRequests(requests);
         g_array_free(requests, TRUE);
-        requests = listDisplayRequests();
-        printDisplayRequests(requests);
-        g_array_free(requests, TRUE);
     }
     else if (!strcmp(argv[1], "stats")) {
         requests = getSysRequestStats();
         printSysRequestStats(requests);
         g_array_free(requests, TRUE);
-
-        requests = getDispRequestStats();
-        printDispRequestStats(requests);
-        g_array_free(requests, TRUE);
     }
     else if (!strcmp(argv[1],"active-nc")) {
         requestSysState("active-nc", POWERD_SYS_STATE_ACTIVE,&cookie);
@@ -1086,20 +838,37 @@ int main (int argc, char **argv)
         return 0;
     }
     else if (!strcmp(argv[1],"display")) {
-        if (argc < 3) {
-            fprintf(stderr,"display requires other arguments\n");
-            usage(argv[0]);
+        int disp_cookie;
+        cli_debug("Requesting Display On");
+        if (requestDisplayState(&disp_cookie) == FALSE) {
+            fprintf(stderr,"failed display request\n");
             return -1;
         }
-        if (buildDisplayRequest(argv,argc,&pdr) == FALSE) {
-            usage(argv[0]);
-            return -1;
-        }
-        requestDisplayState(pdr, "disp-test", &cookie);
-        printf("Display State requested, cookie is %s.\nPress ctrl-c to exit.\n",
-            cookie);
+        printf("Display State requested, cookie is %d.\nPress ctrl-c to exit.\n",
+            disp_cookie);
         pause();  /* wait for SIGINT */
-        clearDisplayState(cookie);
+        clearDisplayState(disp_cookie);
+        return 0;
+    }
+    else if (!strcmp(argv[1],"proximity")) {
+        gboolean enable = TRUE;
+        if (argc != 3) {
+            fprintf(stderr,"proximity requires a parameter: 'on'|'off'\n");
+            return -1;
+        }
+        if (!strcmp(argv[2],"on"))
+            enable = TRUE;
+        else if (!strcmp(argv[2],"off"))
+            enable = FALSE;
+        else {
+            fprintf(stderr,"proximity requires a parameter: 'on'|'off'\n");
+            return -1;
+        }
+
+        if (requestProximityBlanking(enable) == FALSE) {
+            fprintf(stderr,"failed powerd proximity request\n");
+            return -1;
+        }
         return 0;
     }
     else if (!strncmp(argv[1],"clear",strlen("clear"))) {
@@ -1114,8 +883,8 @@ int main (int argc, char **argv)
                 if (!strcmp(argv[1],"clear-sys")) {
                     clearSysState(argv[2]);
                 }
-                else if (!strcmp(argv[1],"clear-disp")) {
-                    clearDisplayState(argv[2]);
+                else if (!g_strcmp0(argv[1], "clear-wakeup")) {
+                    clearWakeup(argv[2]);
                 }
                 else {
                     // Note: We should never get here due to earlier checks
@@ -1144,7 +913,6 @@ int main (int argc, char **argv)
 
         runDbusNameVanishTests(argv[0]);
         runSysTests();
-        runDisplayTests();
 
         g_object_unref(powerd_cli_bus);
         powerd_cli_bus = NULL;
@@ -1163,11 +931,6 @@ int main (int argc, char **argv)
         requestSysState("dbusnametest1", POWERD_SYS_STATE_ACTIVE,&cookie);
         requestSysState("dbusnametest2", POWERD_SYS_STATE_ACTIVE,&cookie);
         requestSysState("dbusnametest3", POWERD_SYS_STATE_ACTIVE,&cookie);
-        pdr.state = POWERD_DISPLAY_STATE_DONT_CARE;
-        pdr.flags = 0;
-        requestDisplayState(pdr, "dbusnametest1", &cookie);
-        requestDisplayState(pdr, "dbusnametest2", &cookie);
-        requestDisplayState(pdr, "dbusnametest3", &cookie);
         g_object_unref(bus);
         // Exit here without cleanup
         return 0;
@@ -1191,13 +954,14 @@ int main (int argc, char **argv)
         }
         setUserAutobrightness(enable);
     } else if (!strcmp(argv[1], "brightness-params")) {
-        int min, max, dflt, ab;
-        if (getBrightnessParams(&min, &max, &dflt, &ab)) {
-            printf("Minimum Brightness: %d\n"
+        int dim, min, max, dflt, ab;
+        if (getBrightnessParams(&dim, &min, &max, &dflt, &ab)) {
+            printf("Dim Brightness: %d\n"
+                   "Minimum Brightness: %d\n"
                    "Maximum Brightness: %d\n"
                    "Default Brightness: %d\n"
                    "Autobrightness:     %ssupported\n",
-                   min, max, dflt, ab ? "" : "not ");
+                   dim, min, max, dflt, ab ? "" : "not ");
         }
     } else if (!strcmp(argv[1], "brightness")) {
         long brightness;
@@ -1215,6 +979,20 @@ int main (int argc, char **argv)
             exit(-1);
         }
         setUserBrightness((int)brightness);
+    } else if (!g_strcmp0(argv[1], "wakeup")) {
+        int n_seconds;
+        powerd_cookie_t cookie;
+        if (argc != 3) {
+            usage(argv[0]);
+            exit(-1);
+        }
+        n_seconds = atoi(argv[2]);
+        if (n_seconds < 1) {
+            fprintf(stderr, "Invalid number of seconds %s\n", argv[2]);
+            usage(argv[0]);
+            exit(-1);
+        }
+        requestWakeup(argv[0], time(NULL)+n_seconds, cookie);
     }
     return 0;
 }
