@@ -39,6 +39,29 @@ void powerd_log(int level, const char *fmt, ...)
         __attribute__ ((format (printf, 2, 3)));
 void powerd_log_init(void);
 
+#define powerd_warn_if_fail(expr) \
+  do { \
+    if (!(expr)) \
+        powerd_warn("%s", #expr); \
+  } while (0)
+
+#define powerd_return_if_fail(expr) \
+  do { \
+    if (!(expr)) { \
+        powerd_warn("%s: assertion '%s' failed", __func__,  #expr); \
+        return; \
+    } \
+  } while (0)
+
+#define powerd_return_val_if_fail(expr,val) \
+  do { \
+    if (!(expr)) { \
+        powerd_warn("%s: assertion '%s' failed", __func__,  #expr); \
+        return val; \
+    } \
+  } while (0)
+
+
 #ifdef __cplusplus
 }
 #endif
