@@ -43,6 +43,7 @@ struct SysStateRequest {
     const char *owner;
     uuid_t cookie;
     enum SysPowerStates state;
+    int power_state_flags;
 };
 
 /*
@@ -120,7 +121,7 @@ static void block_suspend(void)
     int ret;
 
     if (g_atomic_int_add(&suspend_block_count, 1) == 0) {
-        powerd_debug("Blocking suspend");
+        powerd_debug("libsuspend: acquire_wake_lock: %s", power_request_wakelock_name);
         ret = libsuspend_acquire_wake_lock(power_request_wakelock_name);
         if (ret)
             powerd_warn("Could not acquire wake lock");
@@ -142,7 +143,7 @@ static void unblock_suspend(void)
         retry = !g_atomic_int_compare_and_exchange(&suspend_block_count,
                                                    old_count, old_count - 1);
         if (!retry && old_count == 1) {
-            powerd_debug("Unblocking suspend");
+            powerd_debug("libsuspend: release_wake_lock: %s", power_request_wakelock_name);
             ret = libsuspend_release_wake_lock(power_request_wakelock_name);
             if (ret)
                 powerd_warn("Could not release wake lock");
@@ -361,6 +362,7 @@ handle_clear_sys_state(PowerdSource *obj, GDBusMethodInvocation *invocation,
 
 static int enter_suspend(void)
 {
+    powerd_debug("libsuspend: enter_suspend.");
     int ret = libsuspend_enter_suspend();
     if (!ret)
         suspend_active = TRUE;
@@ -369,10 +371,10 @@ static int enter_suspend(void)
 
 static int exit_suspend(void)
 {
+    powerd_debug("libsuspend: exit_suspend.");
     int ret = libsuspend_exit_suspend();
     if (!ret) {
         suspend_active = FALSE;
-        powerd_display_exit_suspend();
     }
     return ret;
 }
@@ -432,11 +434,12 @@ static enum state_transition_state process_state_dequeue(guint *delay)
 
     /* Must call libsuspend_prepare_suspend() _before_ emitting signal */
     if (pending_system_state == POWERD_SYS_STATE_SUSPEND) {
-        powerd_debug("preparing for suspend");
+        powerd_debug("libsuspend: prepare_suspend.");
         ret = libsuspend_prepare_suspend();
         if (ret)
             powerd_warn("Failed to prepare for suspend: %d", ret);
     }
+
     ack_pending = powerd_client_transition_start(pending_system_state);
     powerd_sys_state_signal_emit(pending_system_state);
 
@@ -522,7 +525,6 @@ static void state_transition_process(void)
                                                 delayed_state_transition_process,
                                                 NULL);
     }
-
 }
 
 /* Must be called from main loop */
