@@ -19,8 +19,8 @@
 #ifndef __POWERD_INTERNAL_H__
 #define __POWERD_INTERNAL_H__
 
+#include <time.h> /* time_t */
 #include <uuid/uuid.h>
-#include <glib.h>
 #include <gio/gio.h>
 
 #include "powerd-object.h"
@@ -33,17 +33,13 @@ extern "C" {
 /* Length of a UUID string, including trailing nul */
 #define UUID_STR_LEN 37
 
-struct powerd_display_request {
-    uuid_t cookie;
-    enum powerd_display_state state;
-    guint32 flags;
-};
-
-enum powerd_override_reason {
-    POWERD_OVERRIDE_REASON_POWER_BUTTON,
-    POWERD_OVERRIDE_REASON_PROXIMITY,
-
-    POWERD_NUM_OVERRIDE_REASONS
+enum DisplayStateChangeReason
+{
+    UNITY_SCREEN_REASON_NORMAL = 0,
+    UNITY_SCREEN_REASON_PROXIMITY = 3,
+    UNITY_SCREEN_REASON_NOTIFICATION = 4,
+    UNITY_SCREEN_REASON_SNAP_DECISION = 5,
+    UNITY_SCREEN_REASON_CALL_DONE = 6,
 };
 
 struct DbusNameWatch {
@@ -64,6 +60,10 @@ void powerd_reset_activity_timer(int add);
 void powerd_dbus_init_complete(void);
 int powerd_is_mainloop(void);
 
+/* Display functions */
+gboolean turn_display_on(gboolean on, enum DisplayStateChangeReason reason);
+void keep_display_on();
+
 /* Autobrightness functions */
 void powerd_new_als_event(double lux);
 void powerd_autobrightness_enable(void);
@@ -79,19 +79,11 @@ int powerd_get_brightness(void);
 int powerd_get_max_brightness(void);
 int powerd_set_brightness(int brightness);
 int powerd_set_user_brightness(int brightness);
-void powerd_dim_screen(void);
 gboolean handle_get_brightness_params(PowerdSource *obj,
                                       GDBusMethodInvocation *invocation);
 
-/* Display functions */
 void powerd_brightness_set_value(gint value);
-gboolean powerd_display_enabled(void);
-gboolean display_set_power_mode(int display, const char *powerd_mode);
-void powerd_set_display_state(struct powerd_display_request *req);
-int powerd_display_init(void);
 void powerd_proximity_event(gboolean near);
-void powerd_display_set_override(enum powerd_override_reason reason);
-void powerd_display_clear_override(enum powerd_override_reason reason);
 gboolean handle_user_autobrightness_enable(PowerdSource *obj,
                                            GDBusMethodInvocation *invocation,
                                            gboolean enable);
@@ -99,28 +91,6 @@ gboolean handle_set_user_brightness(PowerdSource *obj,
                                     GDBusMethodInvocation *invocation,
                                     gint brightness);
 
-/* Display request functions */
-int powerd_add_display_request(struct powerd_display_request *request,
-                               const char *name);
-int powerd_update_display_request(struct powerd_display_request *request);
-int powerd_remove_display_request(uuid_t cookie);
-void display_request_init(void);
-void display_request_deinit(void);
-gboolean handle_add_display_request(PowerdSource *obj,
-                                    GDBusMethodInvocation *invocation,
-                                    const char *name, int state, guint32 flags);
-gboolean handle_update_display_request(PowerdSource *obj,
-                                       GDBusMethodInvocation *invocation,
-                                       const gchar *ext_cookie, int state,
-                                       guint32 flags);
-gboolean handle_clear_display_request(PowerdSource *obj,
-                                      GDBusMethodInvocation *invocation,
-                                      const gchar *ext_cookie);
-gboolean handle_list_display_requests(PowerdSource *obj,
-                                      GDBusMethodInvocation *invocation);
-void clear_disp_state_by_owner(const char *owner);
-void powerd_display_exit_suspend(void);
-
 /* System power state requests */
 gboolean handle_list_sys_requests(PowerdSource *obj,
                                   GDBusMethodInvocation *invocation);
@@ -130,6 +100,12 @@ gboolean handle_request_sys_state(PowerdSource *obj,
 gboolean handle_clear_sys_state(PowerdSource *obj,
                                 GDBusMethodInvocation *invocation,
                                 gchar *cookie);
+gboolean handle_enable_proximity_handling(PowerdSource *obj,
+                                          GDBusMethodInvocation *invocation,
+                                          const char *name);
+gboolean handle_disable_proximity_handling(PowerdSource *obj,
+                                           GDBusMethodInvocation *invocation,
+                                           const char *name);
 gboolean request_sys_state_internal(const char *name, int state,
                                     uuid_t *cookie, const char *owner);
 gboolean clear_sys_state_internal(uuid_t cookie);
@@ -140,8 +116,8 @@ void power_request_deinit(void);
 enum SysPowerStates current_system_power_state(void);
 const gchar * state_to_string(int state);
 void powerd_sys_state_signal_emit(enum SysPowerStates state);
-void powerd_display_state_signal_emit(enum powerd_display_state state,
-                                      guint32 flags);
+void powerd_wakeup_signal_emit(void);
+void powerd_update_bus_brightness(void);
 void clear_sys_state_by_owner(const char *owner);
 gboolean powerd_suspend_active(void);
 
@@ -176,6 +152,8 @@ void on_ofono_messagemanager_signal(GDBusProxy *proxy, gchar *sender_name,
     gchar *signal_name, GVariant *parameters, gpointer user_data);
 void on_ofono_ussd_signal(GDBusProxy *proxy, gchar *sender_name,
     gchar *signal_name, GVariant *parameters, gpointer user_data);
+void on_unity_signal(GDBusProxy *proxy, gchar *sender_name,
+    gchar *signal_name, GVariant *parameters, gpointer user_data);
 
 /* dbus proxy async setup */
 void ofono_manager_proxy_connect_cb(GObject *source_object, GAsyncResult *res,
@@ -184,10 +162,13 @@ void ofono_voicecall_proxy_connect_cb(GObject *source_object, GAsyncResult *res,
     gpointer user_data);
 void ofono_proxy_connect_cb(GObject *source_object, GAsyncResult *res,
     gpointer user_data);
+void unity_proxy_connect_cb(GObject *source_object, GAsyncResult *res,
+    gpointer user_data);
 
 /* Sensor functions */
 void powerd_sensors_proximity_enable(void);
 void powerd_sensors_proximity_disable(void);
+void powerd_sensors_proximity_emit(void);
 void powerd_sensors_als_enable(void);
 void powerd_sensors_als_disable(void);
 
@@ -195,6 +176,12 @@ void powerd_sensors_als_disable(void);
 int powerd_ps_init(void);
 void powerd_ps_deinit(void);
 
+/* Hardware alarm functions */
+void wakeup_init(void);
+void wakeup_deinit(void);
+int wakeup_request(const char *name, time_t time, powerd_cookie_t setme_cookie);
+int wakeup_clear(const powerd_cookie_t cookie);
+
 /* Client functions */
 int powerd_client_register(const char *dbus_name, const char *name);
 void powerd_client_unregister(const char *dbus_name);
@@ -207,19 +194,11 @@ void powerd_client_deinit(void);
 /* Statistics functions */
 void powerd_account_request_sys_state(const char *dbus_name, const char *name);
 void powerd_account_clear_sys_state(const char *dbus_name, const char *name);
-void powerd_account_add_display_req(const char *dbus_name, const char *name,
-                                    const struct powerd_display_request *req);
-void powerd_account_update_display_req(const char *dbus_name,
-                                       const char *name,
-                                       const struct powerd_display_request *req);
-void powerd_account_clear_display_req(const char *dbus_name, const char *name);
 void powerd_log_stats(void);
 int powerd_stats_init(void);
 void powerd_stats_deinit(void);
 gboolean handle_get_sys_request_stats(PowerdSource *obj,
                                       GDBusMethodInvocation *invocation);
-gboolean handle_get_disp_request_stats(PowerdSource *obj,
-                                       GDBusMethodInvocation *invocation);
 
 /* Utility functions */
 int powerd_run_mainloop_sync(int (*func)(gpointer), gpointer data);
