@@ -19,6 +19,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
+#include <inttypes.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -135,11 +136,79 @@ static gboolean handle_ack_state_change(PowerdSource *obj,
     return TRUE;
 }
 
+static gboolean
+handle_request_wakeup(PowerdSource *obj,
+                      GDBusMethodInvocation *invocation,
+                      const char *name,
+                      uint64_t wakeup_time)
+{
+    int err;
+    powerd_cookie_t cookie;
+
+    powerd_debug("handle_requestWakeup from %s - %s, %zu",
+                 g_dbus_method_invocation_get_sender(invocation),
+                 name,
+                 (size_t)wakeup_time);
+
+    if ((err = wakeup_request(name, wakeup_time, cookie)))
+    {
+        g_dbus_method_invocation_return_error_literal(invocation,
+                                                      G_DBUS_ERROR,
+                                                      G_DBUS_ERROR_FAILED,
+                                                      strerror(err));
+    }
+    else
+    {
+        g_dbus_method_invocation_return_value(invocation,
+                                              g_variant_new("(s)", cookie));
+    }
+
+    return TRUE;
+}
+
+static gboolean
+handle_clear_wakeup(PowerdSource *obj,
+                    GDBusMethodInvocation *invocation,
+                    const char* cookie)
+{
+    int err;
+
+    powerd_debug("handle_clearWakeup from %s, cookie: %s",
+                 g_dbus_method_invocation_get_sender(invocation),
+                 cookie);
+  
+    if ((err = wakeup_clear(cookie)))
+    {
+        g_dbus_method_invocation_return_error_literal(invocation,
+                                                      G_DBUS_ERROR,
+                                                      G_DBUS_ERROR_FAILED,
+                                                      strerror(err));
+    }
+    else
+    {
+        g_dbus_method_invocation_return_value(invocation, NULL);
+    }
+
+    return TRUE;
+}
+
+/* update the bus property from our brightness */
+void
+powerd_update_bus_brightness(void)
+{
+    if ((powerd_source != NULL) && (powerd_source->priv->skel != NULL)) {
+        com_canonical_powerd_set_brightness(
+            powerd_source->priv->skel,
+            powerd_get_brightness());
+    }
+}
+
 void
 powerd_bus_acquired_cb (GDBusConnection *connection,
                         const gchar     *name,
                         gpointer         user_data)
 {
+    GObject * oskel;
     GError *error = NULL;
 
     powerd_debug("Bus acquired (guid %s)", g_dbus_connection_get_guid (connection));
@@ -156,38 +225,41 @@ powerd_bus_acquired_cb (GDBusConnection *connection,
         powerd_exit(-1);
     }
 
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-request-sys-state",
-        G_CALLBACK(handle_request_sys_state), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-clear-sys-state",
-        G_CALLBACK(handle_clear_sys_state), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-list-sys-requests",
-        G_CALLBACK(handle_list_sys_requests), powerd_source);
+    oskel = G_OBJECT(powerd_source->priv->skel);
 
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-request-display-state",
-        G_CALLBACK(handle_add_display_request), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-update-display-state",
-        G_CALLBACK(handle_update_display_request), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-clear-display-state",
-        G_CALLBACK(handle_clear_display_request), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-register-client",
+    g_signal_connect(oskel, "handle-request-sys-state",
+        G_CALLBACK(handle_request_sys_state), powerd_source);
+    g_signal_connect(oskel, "handle-clear-sys-state",
+        G_CALLBACK(handle_clear_sys_state), powerd_source);
+    g_signal_connect(oskel, "handle-list-sys-requests",
+        G_CALLBACK(handle_list_sys_requests), powerd_source);
+    g_signal_connect(oskel, "handle-enable-proximity-handling",
+        G_CALLBACK(handle_enable_proximity_handling), powerd_source);
+    g_signal_connect(oskel, "handle-disable-proximity-handling",
+        G_CALLBACK(handle_disable_proximity_handling), powerd_source);
+
+    g_signal_connect(oskel, "handle-register-client",
         G_CALLBACK(handle_register_client), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-unregister-client",
+    g_signal_connect(oskel, "handle-unregister-client",
         G_CALLBACK(handle_unregister_client), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-ack-state-change",
+    g_signal_connect(oskel, "handle-ack-state-change",
         G_CALLBACK(handle_ack_state_change), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-list-display-requests",
-        G_CALLBACK(handle_list_display_requests), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-get-sys-request-stats",
+    g_signal_connect(oskel, "handle-get-sys-request-stats",
         G_CALLBACK(handle_get_sys_request_stats), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-get-disp-request-stats",
-        G_CALLBACK(handle_get_disp_request_stats), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-user-autobrightness-enable",
+    g_signal_connect(oskel, "handle-user-autobrightness-enable",
         G_CALLBACK(handle_user_autobrightness_enable), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-get-brightness-params",
+    g_signal_connect(oskel, "handle-get-brightness-params",
         G_CALLBACK(handle_get_brightness_params), powerd_source);
-    g_signal_connect(G_OBJECT(powerd_source->priv->skel), "handle-set-user-brightness",
+    g_signal_connect(oskel, "handle-set-user-brightness",
         G_CALLBACK(handle_set_user_brightness), powerd_source);
 
+    g_signal_connect(oskel, "handle-request-wakeup",
+        G_CALLBACK(handle_request_wakeup), powerd_source);
+    g_signal_connect(oskel, "handle-clear-wakeup",
+        G_CALLBACK(handle_clear_wakeup), powerd_source);
+
+    powerd_update_bus_brightness();
+
     powerd_dbus_init_complete();
 }
 
@@ -234,8 +306,7 @@ powerd_sys_state_signal_emit (enum SysPowerStates state)
 }
 
 void
-powerd_display_state_signal_emit(enum powerd_display_state state,
-                                 guint32 flags)
+powerd_wakeup_signal_emit (void)
 {
     GError *error = NULL;
 
@@ -243,18 +314,18 @@ powerd_display_state_signal_emit(enum powerd_display_state state,
     if (!powerd_source)
         return;
 
-    powerd_debug("Emitting signal for display state change: state=%d flags=0x%08x",
-                 state, flags);
+    powerd_debug("Emitting wakeup signal");
+
     g_dbus_connection_emit_signal(
             POWERD_SOURCE_GET_PRIVATE(powerd_source)->system_bus,
             NULL, /* destination */
             "/com/canonical/powerd",
             "com.canonical.powerd",
-            "DisplayPowerStateChange",
-            g_variant_new("(iu)", state, flags),
+            "Wakeup",
+            NULL,
             &error);
     if (error) {
-        powerd_warn("Unable to signal a state change update: %s", error->message);
+        powerd_warn("Unable to emit wakeup signal: %s", error->message);
         g_error_free(error);
     }
 }
@@ -266,6 +337,7 @@ ofono_manager_proxy_connect_cb(GObject *source_object,
 {
     GError *error = NULL;
     GDBusProxy *ofono_proxy;
+    GDBusProxy **proxy = user_data;
 
     ofono_proxy = g_dbus_proxy_new_finish(res, &error);
     if (error) {
@@ -282,35 +354,8 @@ ofono_manager_proxy_connect_cb(GObject *source_object,
     g_dbus_proxy_call(ofono_proxy, "GetModems", NULL,
                         G_DBUS_CALL_FLAGS_NONE, -1, NULL,
                         ofono_get_modems_cb, NULL);
-}
 
-void
-ofono_voicecall_proxy_connect_cb(GObject *source_object,
-               GAsyncResult *res,
-               gpointer user_data)
-{
-    GError *error = NULL;
-    GDBusProxy *ofono_proxy;
-    struct call_data *call;
-
-    ofono_proxy = g_dbus_proxy_new_finish(res, &error);
-    if (error) {
-        powerd_warn("%s failed: %s", __func__, error->message);
-        g_error_free(error);
-        return;
-    }
-
-    call = user_data;
-    call->ofono_proxy = ofono_proxy;
-
-    /* Register for voicecall signals */
-    g_signal_connect(ofono_proxy, "g-signal",
-                        G_CALLBACK(on_ofono_voicecall_signal), NULL);
-
-    /* Get current voice call state */
-    g_dbus_proxy_call(ofono_proxy, "GetProperties", NULL,
-                        G_DBUS_CALL_FLAGS_NONE, -1, NULL,
-                        ofono_voicecall_get_props_cb, NULL);
+    *proxy = ofono_proxy;
 }
 
 void
@@ -321,6 +366,7 @@ ofono_proxy_connect_cb(GObject *source_object,
     GError *error = NULL;
     GDBusProxy *ofono_proxy = NULL;
     const char *interface_name = "";
+    GDBusProxy **proxy = user_data;
 
     ofono_proxy = g_dbus_proxy_new_finish (res, &error);
     if (error) {
@@ -342,11 +388,40 @@ ofono_proxy_connect_cb(GObject *source_object,
             g_signal_connect(ofono_proxy, "g-signal",
                 G_CALLBACK (on_ofono_ussd_signal), NULL);
         }
-        else {
-            powerd_warn("unknown interface name for this proxy, ignoring");
-        }
         powerd_debug("ofono_proxy_connect_cb succeeded");
+
+        *proxy = ofono_proxy;
     }
+
+}
+
+void unity_proxy_connect_cb(GObject *source_object, GAsyncResult *res,
+                            gpointer user_data)
+{
+    GError *error = NULL;
+    GDBusProxy *unity_proxy;
+    GDBusProxy **proxy = user_data;
+
+    powerd_debug("unity_proxy_connect_cb");
+
+    unity_proxy = g_dbus_proxy_new_finish(res, &error);
+    if (error) {
+        powerd_warn("%s failed: %s", __func__, error->message);
+        g_error_free(error);
+        return;
+    }
+
+    /* Register for display power changes */
+    g_signal_connect(unity_proxy, "g-signal",
+                        G_CALLBACK(on_unity_signal), NULL);
+
+    *proxy = unity_proxy;
+}
+
+static gboolean delayed_clear_request(gpointer userdata)
+{
+    clear_sys_state_by_owner((gchar*) userdata);
+    return FALSE;
 }
 
 void
@@ -355,8 +430,7 @@ powerd_name_vanished_cb(GDBusConnection *connection,
                         gpointer user_data)
 {
     powerd_warn("%s vanished from dbus, clearing associated requests", name);
-    clear_sys_state_by_owner(name);
-    clear_disp_state_by_owner(name);
+    g_timeout_add(5000, delayed_clear_request, (gpointer) name);
     powerd_client_unregister(name);
     // This can return false if the hash entry was removed because the
     // last request was dropped, in other words, this call will fail
